Attribute VB_Name = "MInitDirectX"
Option Explicit
'Credits to Richard Clark (http://www.c2i.fr) for his
'rotating code

Public DX As DirectX7
Public DD As DirectDraw7
Public lngAlpha As Long
Public surfPrimary As DirectDrawSurface7
Public surfBackBuffer As DirectDrawSurface7
Public di As DirectInput
Public diDEV As DirectInputDevice
Public diState As DIKEYBOARDSTATE
Public iKeyCounter As Integer
Public speed
Public carx As Long, cary As Long
Public Const PI As Double = 3.14159265358979
Public debugmode As Boolean
Public map(10, 10, 2) As Integer
Public carwidth, carheight


Public Function InitDirectX() As Boolean
On Error GoTo err
    
InitDirectX = True
Set DX = New DirectX7

Exit Function

err:
InitDirectX = False
End Function

Public Function InitDirectDraw(ByVal frm As Form, ByVal lngWidth As Long, ByVal lngHeight As Long, lngBitColor As Long) As Boolean
On Error GoTo err

InitDirectDraw = True
Set DD = DX.DirectDrawCreate("")

frm.Show
    
DD.SetCooperativeLevel frm.hWnd, DDSCL_FULLSCREEN Or DDSCL_EXCLUSIVE
DD.SetDisplayMode lngWidth, lngHeight, lngBitColor, 0, DDSDM_DEFAULT
    
Dim ddsdTemp As DDSURFACEDESC2
ddsdTemp.lFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
ddsdTemp.ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX
ddsdTemp.lBackBufferCount = 1
Set surfPrimary = DD.CreateSurface(ddsdTemp)

Dim caps As DDSCAPS2
caps.lCaps = DDSCAPS_BACKBUFFER
Set surfBackBuffer = surfPrimary.GetAttachedSurface(caps)

Exit Function

err:
InitDirectDraw = False
End Function

Public Function InitSurfaceOffScreen(ByRef surf As DirectDrawSurface7, sFile As String, Optional lWidth As Long, Optional lHeight As Long) As Boolean
Dim ddsd As DDSURFACEDESC2
On Error GoTo err
InitSurfaceOffScreen = True
ddsd.lFlags = DDSD_CAPS Or DDSD_WIDTH Or DDSD_HEIGHT
ddsd.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
If sFile <> "" Then
    Loaddirectxsurface surf, sFile, lWidth, lHeight
Else
    ddsd.lHeight = lHeight
    ddsd.lWidth = lWidth
    Set surf = DD.CreateSurface(ddsd)
End If
Exit Function
err:
InitSurfaceOffScreen = False
Debug.Print err.Number
End Function

Public Function Terminate_DirectX()
Set surfBackBuffer = Nothing
Set surfPrimary = Nothing
Set DD = Nothing
Set DX = Nothing
End Function


Public Function Transparence(ByRef surfDest As DirectDrawSurface7, ByRef surfSource As DirectDrawSurface7, ByVal sngAlpha As Single, lngXDest As Long, lngYDest As Long)
Dim lWidth As Long, lHeight As Long
Dim ddsdDest As DDSURFACEDESC2, ddsdSource As DDSURFACEDESC2
Dim lColorV As Long, lColorB As Long, lNewColor As Long
Dim rEmpty As RECT, rEmpty2 As RECT, iX As Long, iY As Long

surfSource.GetSurfaceDesc ddsdSource
surfDest.GetSurfaceDesc ddsdDest
lWidth = ddsdSource.lWidth
lHeight = ddsdSource.lHeight

surfSource.Lock rEmpty, ddsdSource, DDLOCK_WAIT, 0
surfDest.Lock rEmpty2, ddsdDest, DDLOCK_WAIT, 0
For iY = 0 To lHeight - 1
    For iX = 0 To lWidth - 1
        lColorV = surfSource.GetLockedPixel(iX, iY)
        If lColorV <> 0 Then
            lColorB = surfDest.GetLockedPixel(lngXDest + iX, lngYDest + iY)
            lNewColor = ddsdDest.ddpfPixelFormat.lRBitMask And (lColorV And ddsdSource.ddpfPixelFormat.lRBitMask) * sngAlpha + (lColorB And ddsdDest.ddpfPixelFormat.lRBitMask) * (1 - sngAlpha)
            lNewColor = lNewColor Or (ddsdDest.ddpfPixelFormat.lGBitMask And (lColorV And ddsdSource.ddpfPixelFormat.lGBitMask) * sngAlpha + (lColorB And ddsdDest.ddpfPixelFormat.lGBitMask) * (1 - sngAlpha))
            lNewColor = lNewColor Or (ddsdDest.ddpfPixelFormat.lBBitMask And (lColorV And ddsdSource.ddpfPixelFormat.lBBitMask) * sngAlpha + (lColorB And ddsdDest.ddpfPixelFormat.lBBitMask) * (1 - sngAlpha))
            surfDest.SetLockedPixel lngXDest + iX, lngYDest + iY, lNewColor
        End If
    Next iX
Next iY
surfSource.Unlock rEmpty
surfDest.Unlock rEmpty2

End Function

Public Function RotateSurface(ByRef surfSource As DirectDrawSurface7, surfDestination As DirectDrawSurface7, lngAngle As Long, XDest As Long, Ydest As Long)
Dim ddsdOrigine As DDSURFACEDESC2, ddsdDestination As DDSURFACEDESC2
Dim iX As Long, iY As Long
Dim iXDest As Long, iYDest As Long
Dim rEmpty As RECT, rEmpty2 As RECT
Dim sngA As Single, SinA As Single, CosA As Single
Dim dblRMax As Long
Dim lngXO As Long, lngYO As Long
Dim lngColor As Long
Dim lWidth As Long, lHeight As Long

sngA = lngAngle * PI / 180
SinA = Sin(sngA)
CosA = Cos(sngA)

surfSource.GetSurfaceDesc ddsdOrigine
lWidth = ddsdOrigine.lWidth
lHeight = ddsdOrigine.lHeight
dblRMax = Sqr(lWidth ^ 2 + lHeight ^ 2)
surfSource.Lock rEmpty, ddsdOrigine, DDLOCK_WAIT, 0
surfDestination.GetSurfaceDesc ddsdDestination
surfDestination.Lock rEmpty2, ddsdDestination, DDLOCK_WAIT, 0

XDest = XDest + lWidth / 2
Ydest = Ydest + lHeight / 2
For iX = -dblRMax To dblRMax
    For iY = -dblRMax To dblRMax
        lngXO = lWidth / 2 - (iX * CosA + iY * SinA)
        lngYO = lHeight / 2 - (iX * SinA - iY * CosA)
        If lngXO >= 0 Then
            If lngYO >= 0 Then
                If lngXO < lWidth Then
                    If lngYO < lHeight Then
                        lngColor = surfSource.GetLockedPixel(lngXO, lngYO)
                        If lngColor <> 0 Then
                            surfDestination.SetLockedPixel XDest + iX, Ydest + iY, lngColor
                        End If
                    End If
                End If
            End If
        End If
    Next iY
Next iX
surfSource.Unlock rEmpty
surfDestination.Unlock rEmpty2
carwidth = lWidth / 2 - (iX * CosA + iY * SinA)
carheight = lHeight / 2 - (iX * SinA - iY * CosA)
End Function


Sub check_keys()
diDEV.GetDeviceStateKeyboard diState

For iKeyCounter = 0 To 255
If diState.key(iKeyCounter) <> 0 Then
Dim alpharad As Long
Select Case iKeyCounter
    Case 1  'escape
Form1.Terminate
    Case 28 'return on keyboard
    Case 57 'space bar
    If CInt(speed) < 0 Then
    speed = speed + 1.5
    ElseIf CInt(speed) > 0 Then
    speed = speed - 1.5
    ElseIf CInt(speed) = 0 Then
    speed = 0
    End If
    Case 59 'F1 info
    debugmode = Not debugmode
    Case 60 'F2
    Case 200 'up
speed = speed + 1
        Case 203 'Left
    lngAlpha = lngAlpha - 5
    Case 205 'Right
    lngAlpha = lngAlpha + 5
    Case 208 'down
speed = speed - 1
        End Select
End If
Next
End Sub

